@@ -1,4 +1,5 @@
 use libc;
+use std::ffi::CString;
 use std::process::exit;
 
 pub enum Fork {
@@ -10,10 +11,10 @@ pub enum Fork {
 // and returns the process ID of the child process to the parent process.
 // Otherwise, a value of -1 is returned to the parent process, no child process
 // is created.
-pub fn fork() -> Result<Fork, libc::pid_t> {
+pub fn fork() -> Result<Fork, ()> {
     let res = unsafe { libc::fork() };
     match res {
-        -1 => Err(res),
+        -1 => Err(()),
         0 => Ok(Fork::Child),
         res => Ok(Fork::Parent(res)),
     }
@@ -22,10 +23,22 @@ pub fn fork() -> Result<Fork, libc::pid_t> {
 // Upon successful completion, the setsid() system call returns the value of the
 // process group ID of the new process group, which is the same as the process ID
 // of the calling process. If an error occurs, setsid() returns -1
-pub fn setsid() -> Result<libc::pid_t, libc::pid_t> {
+pub fn setsid() -> Result<libc::pid_t, ()> {
     let res = unsafe { libc::setsid() };
     match res {
-        -1 => Err(res),
+        -1 => Err(()),
+        res => Ok(res),
+    }
+}
+
+// Upon successful completion, 0 shall be returned. Otherwise, -1 shall be
+// returned, the current working directory shall remain unchanged, and errno
+// shall be set to indicate the error.
+pub fn chdir() -> Result<libc::c_int, ()> {
+    let dir = CString::new("/").expect("CString::new failed");
+    let res = unsafe { libc::chdir(dir.as_ptr()) };
+    match res {
+        -1 => Err(()),
         res => Ok(res),
     }
 }
@@ -36,13 +49,17 @@ pub fn setsid() -> Result<libc::pid_t, libc::pid_t> {
 // The child forks a grandchild
 // The child exits
 // The grandchild is now the daemon
-pub fn daemon() -> Result<Fork, libc::pid_t> {
+// nochdir = false, changes the current working directory to the root (/).
+pub fn daemon(nochdir: bool) -> Result<Fork, ()> {
     match fork() {
         Ok(Fork::Parent(_)) => exit(0),
-        Ok(Fork::Child) => match setsid() {
-            Ok(_) => fork(),
-            Err(n) => Err(n),
-        },
+        Ok(Fork::Child) => setsid().and_then(|_| {
+            if nochdir {
+                fork()
+            } else {
+                chdir().and_then(|_| fork())
+            }
+        }),
         Err(n) => Err(n),
     }
 }
@@ -53,10 +70,8 @@ mod tests {
 
     #[test]
     fn test_fork() {
-        match fork() {
-            Ok(Fork::Parent(child)) => assert!(child > 0),
-            Ok(Fork::Child) => println!("child process"),
-            Err(_) => assert!(false),
+        if let Ok(Fork::Parent(child)) = fork() {
+            assert!(child > 0);
         }
     }
 }
